@@ -1,11 +1,13 @@
 import argparse
+from typing import List
 
 from tabulate import tabulate
 
 from xklb import db, utils
+from xklb.utils import log
 
 
-def get_table(args):
+def get_table(args) -> List[dict]:
     cols = args.db["media"].columns
 
     db_resp = list(
@@ -14,7 +16,8 @@ def get_table(args):
         select path, size
         from media
         where 1=1
-        {'and is_deleted = 0' if 'is_deleted' in cols else ''}
+            {'and is_deleted = 0' if 'is_deleted' in cols else ''}
+            {'and is_downloaded = 0' if 'is_downloaded' in cols else ''}
         order by path
         """
         )
@@ -23,7 +26,7 @@ def get_table(args):
     d = {}
     for m in db_resp:
         p = m["path"].split("/")
-        while len(p) > 2:
+        while len(p) >= 3:
             p.pop()
             parent = "/".join(p) + "/"
 
@@ -34,7 +37,7 @@ def get_table(args):
                 d[parent] = dict(size=m["size"], count=1)
 
     for path, pdict in list(d.items()):
-        if pdict["count"] < 35 or pdict["count"] > 3500:
+        if pdict["count"] < args.lower or pdict["count"] > args.upper:
             d.pop(path)
 
     d = [{**v, "path": k} for k, v in d.items()]
@@ -42,17 +45,20 @@ def get_table(args):
     return sorted(d, key=lambda x: x["size"])
 
 
-def parse_args():
+def parse_args() -> argparse.Namespace:
     parser = argparse.ArgumentParser()
     parser.add_argument("database")
     parser.add_argument("--limit", "-L", "-l", "-queue", "--queue")
+    parser.add_argument("--lower", default=35, type=int)
+    parser.add_argument("--upper", default=3500, type=int)
     parser.add_argument("--verbose", "-v", action="count", default=0)
     args = parser.parse_args()
     args.db = db.connect(args)
+    log.info(utils.dict_filter_bool(args.__dict__))
     return args
 
 
-def large_folders():
+def large_folders() -> None:
     args = parse_args()
     tbl = get_table(args)
 
