@@ -1,5 +1,8 @@
 import argparse, math, os, sys
+from multiprocessing import TimeoutError
 from pathlib import Path
+from timeit import default_timer as timer
+from typing import Dict, List, Union
 
 from joblib import Parallel, delayed
 
@@ -8,7 +11,15 @@ from xklb.player import mark_media_deleted
 from xklb.utils import SQLITE_PARAM_LIMIT, log
 
 
-def calculate_sparseness(stat):
+class DBType:
+    audio = "audio"
+    video = "video"
+    filesystem = "filesystem"
+    text = "text"
+    image = "image"
+
+
+def calculate_sparseness(stat) -> int:
     if stat.st_size == 0:
         sparseness = 0
     else:
@@ -17,12 +28,13 @@ def calculate_sparseness(stat):
     return sparseness
 
 
-def extract_metadata(args, f):
+def extract_metadata(mp_args, f) -> Union[Dict[str, int], None]:
     log.debug(f)
 
     try:
         stat = os.stat(f)
     except Exception:
+        log.error(f"[{f}] Could not read file stats (possible filesystem corruption; check dmesg)")
         return
 
     media = dict(
@@ -38,46 +50,58 @@ def extract_metadata(args, f):
     if hasattr(stat, "st_blocks"):
         media = {**media, "sparseness": calculate_sparseness(stat)}
 
-    if args.db_type == "f":
+    if mp_args.db_type == DBType.filesystem:
         media = {**media, "is_dir": os.path.isdir(f)}
 
-    if args.db_type == "t":
-        return books.munge_book_tags(media, f)
+    if mp_args.db_type in [DBType.audio, DBType.video]:
+        return av.munge_av_tags(mp_args, media, f)
 
-    if args.db_type in ["a", "v"]:
-        return av.munge_av_tags(args, media, f)
+    if mp_args.db_type == DBType.text:
+        try:
+            start = timer()
+            if any([mp_args.ocr, mp_args.speech_recognition]):
+                media = books.munge_book_tags_slow(media, f)
+            else:
+                media = books.munge_book_tags_fast(media, f)
+        except TimeoutError:
+            log.warning(f"[{f}]: Timed out trying to read file")
+            return media
+        else:
+            log.debug(f"[{f}]: {timer()-start}")
+            return media
 
     return media
 
 
-def extract_chunk(args, l):
+def extract_chunk(args, chunk_paths) -> None:
     n_jobs = -1
-    if args.db_type in ["t", "p", "f"]:
-        n_jobs = (os.cpu_count() or 4) * 2
-    if args.verbose > 0:
+    if args.db_type in [DBType.text, DBType.image, DBType.filesystem]:
+        n_jobs = utils.CPU_COUNT
+    if args.verbose >= 2:
         n_jobs = 1
 
-    mp_args = argparse.Namespace(delete_unplayable=args.delete_unplayable, db_type=args.db_type)
-    metadata = Parallel(n_jobs=n_jobs)(delayed(extract_metadata)(mp_args, file) for file in l) or []
+    mp_args = argparse.Namespace(**{k: v for k, v in args.__dict__.items() if k not in {"db"}})
+    metadata = Parallel(n_jobs=n_jobs)(delayed(extract_metadata)(mp_args, p) for p in chunk_paths) or []
 
-    if args.db_type == "i":
-        metadata = books.extract_image_metadata_chunk(metadata, l)
+    if args.db_type == DBType.image:
+        metadata = books.extract_image_metadata_chunk(metadata, chunk_paths)
 
-    [p.unlink() for p in Path(paths.SUB_TEMP_DIR).glob("*.srt")]
+    if args.scan_subtitles:
+        [p.unlink() for p in Path(paths.SUB_TEMP_DIR).glob("*.srt")]
 
     args.db["media"].insert_all(list(filter(None, metadata)), pk="path", alter=True, replace=True)
 
 
-def find_new_files(args, path):
-    if args.db_type == "a":
+def find_new_files(args, path) -> List[str]:
+    if args.db_type == DBType.audio:
         scanned_files = paths.get_media_files(path, audio=True)
-    elif args.db_type == "v":
+    elif args.db_type == DBType.video:
         scanned_files = paths.get_media_files(path)
-    elif args.db_type == "t":
+    elif args.db_type == DBType.text:
         scanned_files = paths.get_text_files(path, OCR=args.ocr, speech_recognition=args.speech_recognition)
-    elif args.db_type == "i":
+    elif args.db_type == DBType.image:
         scanned_files = paths.get_image_files(path)
-    elif args.db_type == "f":
+    elif args.db_type == DBType.filesystem:
         # thanks to these people for making rglob fast https://bugs.python.org/issue26032
         scanned_files = [str(p) for p in Path(path).resolve().rglob("*")]
     else:
@@ -87,7 +111,17 @@ def find_new_files(args, path):
 
     try:
         existing_set = set(
-            [d["path"] for d in args.db.query(f"select path from media where is_deleted=0 and path like '{path}%'")]
+            [
+                d["path"]
+                for d in args.db.query(
+                    f"""select path from media
+                where 1=1
+                    and is_deleted=0
+                    and path like '{path}%'
+                    {'AND is_downloaded=1' if 'is_downloaded' in args.db['media'].columns else ''}
+                """
+                )
+            ]
         )
     except Exception:
         new_files = list(scanned_set)
@@ -104,9 +138,10 @@ def find_new_files(args, path):
     return new_files
 
 
-def scan_path(args, path):
+def scan_path(args, path) -> int:
     path = Path(path).resolve()
     if not path.exists():
+        print(f"[{path}] Path does not exist")
         return 0
     print(f"[{path}] Building file list...")
 
@@ -116,7 +151,10 @@ def scan_path(args, path):
         print(f"[{path}] Adding {len(new_files)} new media")
         log.debug(new_files)
 
-        batch_count = SQLITE_PARAM_LIMIT // 100
+        if args.db_type in [DBType.text]:
+            batch_count = utils.CPU_COUNT
+        else:
+            batch_count = SQLITE_PARAM_LIMIT // 100
         chunks_count = math.ceil(len(new_files) / batch_count)
         df_chunked = utils.chunks(new_files, batch_count)
         for idx, l in enumerate(df_chunked):
@@ -131,9 +169,7 @@ def scan_path(args, path):
     return len(new_files)
 
 
-def extractor(args):
-    Path(args.database).touch()
-    args.db = db.connect(args)
+def extractor(args) -> None:
     new_files = 0
     for path in args.paths:
         new_files += scan_path(args, path)
@@ -142,7 +178,7 @@ def extractor(args):
         db.optimize(args)
 
 
-def parse_args():
+def parse_args() -> argparse.Namespace:
     parser = argparse.ArgumentParser(
         prog="library fsadd",
         usage="""library fsadd [--audio | --video | --image |  --text | --filesystem] [database] paths ...
@@ -163,6 +199,9 @@ def parse_args():
     Create text database and scan with OCR and speech-recognition:
         library fsadd --text --ocr --speech-recognition ./receipts_and_messages/
 
+    Create video database and read internal/external subtitle files for use in search:
+        library fsadd --scan-subtitles ./tv/
+
     Run with --optimize to add indexes to every int and text column:
         library fsadd --optimize --audio ./music/
 
@@ -170,53 +209,65 @@ def parse_args():
         library fsadd --audio podcasts.db ./podcasts/ ./another/folder/
 """,
     )
-    parser.add_argument("database", nargs="?")
-    parser.add_argument("paths", nargs="+")
-    parser.add_argument("--db", "-db", help=argparse.SUPPRESS)
 
     db_type = parser.add_mutually_exclusive_group()
-    db_type.add_argument("--audio", action="store_const", dest="db_type", const="a", help="Create audio database")
     db_type.add_argument(
-        "--filesystem", action="store_const", dest="db_type", const="f", help="Create filesystem database"
+        "--audio", action="store_const", dest="db_type", const=DBType.audio, help="Create audio database"
     )
-    db_type.add_argument("--video", action="store_const", dest="db_type", const="v", help="Create video database")
-    db_type.add_argument("--text", action="store_const", dest="db_type", const="t", help="Create text database")
-    db_type.add_argument("--image", action="store_const", dest="db_type", const="i", help="Create image database")
-    parser.set_defaults(db_type="v")
+    db_type.add_argument(
+        "--filesystem", action="store_const", dest="db_type", const=DBType.filesystem, help="Create filesystem database"
+    )
+    db_type.add_argument(
+        "--video", action="store_const", dest="db_type", const=DBType.video, help="Create video database"
+    )
+    db_type.add_argument("--text", action="store_const", dest="db_type", const=DBType.text, help="Create text database")
+    db_type.add_argument(
+        "--image", action="store_const", dest="db_type", const=DBType.image, help="Create image database"
+    )
+    parser.set_defaults(db_type=DBType.video)
 
     parser.add_argument("--ocr", "--OCR", action="store_true", help=argparse.SUPPRESS)
     parser.add_argument("--speech-recognition", "--speech", action="store_true", help=argparse.SUPPRESS)
+    parser.add_argument("--scan-subtitles", "--scan-subtitle", action="store_true", help=argparse.SUPPRESS)
     parser.add_argument("--subtitle", action="store_true", help=argparse.SUPPRESS)
     parser.add_argument("--youtube-only", action="store_true", help=argparse.SUPPRESS)
     parser.add_argument("--subliminal-only", action="store_true", help=argparse.SUPPRESS)
     parser.add_argument("--delete-unplayable", action="store_true", help=argparse.SUPPRESS)
     parser.add_argument("--optimize", action="store_true", help=argparse.SUPPRESS)
     parser.add_argument("--verbose", "-v", action="count", default=0)
+    parser.add_argument("--db", "-db", help=argparse.SUPPRESS)
+
+    parser.add_argument("database", nargs="?")
+    parser.add_argument("paths", nargs="+")
     args = parser.parse_args()
 
     if args.db:
         args.database = args.db
 
     if not args.database:
-        if args.db_type == "a":
+        if args.db_type == DBType.audio:
             args.database = "audio.db"
-        elif args.db_type == "f":
+        elif args.db_type == DBType.filesystem:
             args.database = "fs.db"
-        elif args.db_type == "v":
+        elif args.db_type == DBType.video:
             args.database = "video.db"
-        elif args.db_type == "t":
+        elif args.db_type == DBType.text:
             args.database = "text.db"
-        elif args.db_type == "i":
+        elif args.db_type == DBType.image:
             args.database = "image.db"
         else:
             raise Exception(f"fs_extract for db_type {args.db_type} not implemented")
 
+    if args.db:
+        args.database = args.db
+    Path(args.database).touch()
+    args.db = db.connect(args)
     log.info(utils.dict_filter_bool(args.__dict__))
 
     return args
 
 
-def main(args=None):
+def main(args=None) -> None:
     if args:
         sys.argv[1:] = args
 
