@@ -1,11 +1,11 @@
 import argparse
 from time import sleep
-from typing import Dict, List
+from typing import Dict, List, Tuple
 
-from xklb import db
+from xklb import db, utils
 from xklb.player import generic_player, mark_media_watched, override_sort, printer
 from xklb.tabs_extract import Frequency
-from xklb.utils import SC, cmd, dict_filter_bool, flatten, log
+from xklb.utils import SC, cmd, flatten, log
 
 tabs_include_string = (
     lambda x: f"""and (
@@ -24,7 +24,7 @@ tabs_exclude_string = (
 )
 
 
-def construct_tabs_query(args):
+def construct_tabs_query(args) -> Tuple[str, dict]:
     cf = []
     bindings = {}
 
@@ -78,14 +78,14 @@ def construct_tabs_query(args):
     return query, bindings
 
 
-def play(args, m: Dict):
+def play(args, m: Dict) -> None:
     media_file = m["path"]
 
     cmd(*generic_player(args), media_file, strict=False)
     mark_media_watched(args, media_file)
 
 
-def frequency_filter(args, media: List[Dict]):
+def frequency_filter(args, media: List[Dict]) -> List[dict]:
     mapper = {
         Frequency.Daily.value: 1,
         Frequency.Weekly.value: 7,
@@ -110,8 +110,7 @@ def frequency_filter(args, media: List[Dict]):
     return filtered_media
 
 
-def process_tabs_actions(args, construct_tabs_query):
-    args.db = db.connect(args)
+def process_tabs_actions(args, construct_tabs_query) -> None:
     query, bindings = construct_tabs_query(args)
 
     if args.print:
@@ -126,11 +125,11 @@ def process_tabs_actions(args, construct_tabs_query):
 
     for m in media:
         play(args, m)
-    if len(media) > 10:
-        sleep(0.3)
+        if len(media) >= 9:
+            sleep(0.3)
 
 
-def parse_args(action, default_db):
+def parse_args(action, default_db) -> argparse.Namespace:
     parser = argparse.ArgumentParser(
         prog="library tabs",
         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
@@ -201,26 +200,26 @@ def parse_args(action, default_db):
         ╘═══════════════════════════════════════╧═════════════╧══════════════╛
 """,
     )
-    parser.add_argument(
-        "database",
-        nargs="?",
-        default=default_db,
-        help="Database file. If not specified a generic name will be used: audio.db, video.db, fs.db, etc",
-    )
 
     parser.add_argument("--sort", "-u", nargs="+")
     parser.add_argument("--where", "-w", nargs="+", action="extend", default=[])
     parser.add_argument("--include", "-s", "--search", nargs="+", action="extend", default=[])
     parser.add_argument("--exclude", "-E", "-e", nargs="+", action="extend", default=[])
-
     parser.add_argument("--print", "-p", default=False, const="p", nargs="?")
     parser.add_argument("--delete", "--remove", "--erase", "--rm", "-rm", action="store_true", help=argparse.SUPPRESS)
     parser.add_argument("--cols", "-cols", "-col", nargs="*", help="Include a non-standard column when printing")
     parser.add_argument("--limit", "-L", "-l", "-queue", "--queue")
     parser.add_argument("--skip", "-S")
 
-    parser.add_argument("--db", "-db")
     parser.add_argument("--verbose", "-v", action="count", default=0)
+    parser.add_argument("--db", "-db")
+
+    parser.add_argument(
+        "database",
+        nargs="?",
+        default=default_db,
+        help="Database file. If not specified a generic name will be used: audio.db, video.db, fs.db, etc",
+    )
     args = parser.parse_args()
     args.action = action
 
@@ -237,11 +236,14 @@ def parse_args(action, default_db):
     if args.delete:
         args.print += "d"
 
-    log.info(dict_filter_bool(args.__dict__))
+    if args.db:
+        args.database = args.db
+    args.db = db.connect(args)
+    log.info(utils.dict_filter_bool(args.__dict__))
 
     return args
 
 
-def tabs():
+def tabs() -> None:
     args = parse_args(SC.tabs, "tabs.db")
     process_tabs_actions(args, construct_tabs_query)
